// file: $isip/class/algo/Filter/filt_04.cc
// version: $Id: filt_04.cc,v 1.9 2002/05/31 21:57:27 picone Exp $
//

// isip include files
//
#include "Filter.h"

// method: sofSize
//
// arguments: none
//
// return: size of object
//
// this method returns the size of the object in the Sof file and is
// used for binary write
//
long Filter::sofSize() const {

  // start with the space required for the algorithm name
  //
  long bytes = ALGO_MAP.elementSofSize();
  
  // add the space required for the mode name
  //
  bytes += DMODE_MAP.elementSofSize();
  
  // add the space required for the computation mode name
  //
  bytes += CMODE_MAP.elementSofSize();

  // add the space required for the implementation name
  //
  bytes += IMPL_MAP.elementSofSize();

  // add the space required for the coefficients
  //
  bytes += ma_coef_d.sofSize();
  bytes += ar_coef_d.sofSize();
  bytes += ma_lag_d.sofSize();
  bytes += ar_lag_d.sofSize();
  
  // add the space required for the debug level
  //
  bytes += debug_level_d.sofSize();
  
  // return the size
  //
  return bytes;
}

// method: write
//
// arguments:
//  Sof& sof: (input) sof file object
//  long tag: (input) sof object instance tag
//  const String& name: (input) sof object instance name
//
// return: a boolean value indicating status
//
// this method has the object write itself to an Sof file
//
boolean Filter::write(Sof& sof_a, long tag_a, const String& name_a) const {

  // declare local variable
  //
  long obj_size;
  
  // write the instance of the object into the Sof file
  //
  if (sof_a.isText()) {
    
    // set the size to by dynamic
    //
    obj_size = Sof::ANY_SIZE;
  }
  
  else {

    // the size index and the size of each element
    //
    obj_size = sofSize();
  }
  
  // write the object into the sof file's index
  //
  if (!sof_a.put(name_a, tag_a, obj_size)) {
    return false;
  }
  
  // write data and exit gracefully
  //
  return writeData(sof_a);
}

// method: writeData
//
// arguments:
//  Sof& sof: (input) sof file object
//  const String& pname: (input) parameter name
//
// return: a boolean value indicating status
//
// this method branches on the algorithm name
//
boolean Filter::writeData(Sof& sof_a, const String& pname_a) const {
  
  // declare local variables
  //
  boolean status = false;
  
  // write a start string if necessary
  //
  sof_a.writeLabelPrefix(pname_a);

  // write the algorithm name
  //
  ALGO_MAP.writeElementData(sof_a, PARAM_ALGORITHM, (long)algorithm_d);
    
  // check known algorithms: invoke the proper lower-level method
  //
  if (algorithm_d == LTI) {
    status = writeDataCommon(sof_a, pname_a);
  }

  // exit ungracefully
  //
  else {
    return Error::handle(name(), L"writeData",
			 ERR_UNKALG, __FILE__, __LINE__);
  }

  // put an end string if necessary
  //
  sof_a.writeLabelSuffix(pname_a);
  
  // exit gracefully
  //
  return status;
}

// method: writeDataCommon
//
// arguments:
//  Sof& sof: (input) sof file object
//  const String& pname: (input) parameter name
//
// return: a boolean value indicating status
//
// this method writes data to a file for several algorithms that
// share the same parameters
//
boolean Filter::writeDataCommon(Sof& sof_a, const String& pname_a) const {
  
  // write the mode type
  //
  CMODE_MAP.writeElementData(sof_a, PARAM_CMODE, (long)cmode_d);
  
  // write the computation mode type
  //
  DMODE_MAP.writeElementData(sof_a, PARAM_DMODE, (long)dmode_d);
  
  // write the implementation type
  //
  IMPL_MAP.writeElementData(sof_a, PARAM_IMPLEMENTATION,
			    (long)implementation_d);
  
  // write the ma_coef
  //
  ma_coef_d.writeData(sof_a, PARAM_MA_COEF);
 
  // write the ar_coef
  //
  ar_coef_d.writeData(sof_a, PARAM_AR_COEF);
 
  // write the ma_lag
  //
  ma_lag_d.writeData(sof_a, PARAM_MA_LAG);

  // write the ar_lag
  //
  ar_lag_d.writeData(sof_a, PARAM_AR_LAG);

  // write debug level
  //
  debug_level_d.writeData(sof_a, PARAM_DBGL);

  // exit gracefully
  //
  return true;
}
