// file: $isip/class/algo/Prediction/pred_04.cc
// version: $Id: pred_04.cc,v 1.4 2002/02/06 03:16:49 zheng Exp $
//

// isip include files
//
#include "Prediction.h"

// method: sofSize
//
// arguments: none
//
// return: a long value indicating the size of object
//
// this method return  the size of the object
//
long Prediction::sofSize() const {
  
  // calculate the space of data type and algorithm
  //
  long bytes = order_d.sofSize();
  bytes += dyn_range_d.sofSize();
  bytes += ALGO_MAP.elementSofSize();
  bytes += IMPL_MAP.elementSofSize();
  bytes += debug_level_d.sofSize();
  
  // return the size
  //
  return bytes;
}

// method: write
//
// arguments:
//  Sof& sof: (input) sof file object
//  long tag: (input) sof object instance tag
//  const String& name: (input) sof object instance name
//
// return: a boolean value indicating status
//
// this method has the object write itself, with specified name and
// tag, to an Sof file
//
boolean Prediction::write(Sof& sof_a, long tag_a,
			  const String& name_a) const {

  // declare local variables
  //  
  long obj_size;

  // write the instance of the object into the Sof file
  //  
  if (sof_a.isText()) {
    obj_size = Sof::ANY_SIZE;
  }
  else {
    obj_size = sofSize();
  }
  
  // put the object into the sof file's index
  //
  if (!sof_a.put(name_a, tag_a, obj_size)) {
    return false;
  }

  // exit gracefully
  //
  return writeData(sof_a);
}

// method: writeData
//
// arguments:
//  Sof& sof: (input) sof file object
//  const String& pname: (input) textual parameter name
//
// return: logical error status
//
// this method has the object read itself from an Sof file. it assumes
// that the Sof file is already positioned correctly.
//
boolean Prediction::writeData(Sof& sof_a, const String& pname_a) const {

  // declare local variables
  //
  boolean status = false;
  
  // write a start string if necessary
  //
  sof_a.writeLabelPrefix(pname_a);
  
  // write the algorithm type
  //
  ALGO_MAP.writeElementData(sof_a, PARAM_ALGORITHM, (long)algorithm_d);

  // write the implementation type
  //
  IMPL_MAP.writeElementData(sof_a, PARAM_IMPLEMENTATION,
			    (long)implementation_d);

  // invoke the proper lower-level method
  //
  if ((algorithm_d == AUTOCORRELATION) ||
      (algorithm_d == COVARIANCE) ||
      (algorithm_d == LATTICE) ||
      (algorithm_d == REFLECTION) ||
      (algorithm_d == LOG_AREA_RATIO)) {      
    status = writeDataCommon(sof_a, pname_a);
  }
  else {
    return Error::handle(name(), L"writeData", Error::ARG,
			 __FILE__, __LINE__);
  }
  
  // put an end string if necessary
  //
  sof_a.writeLabelSuffix(pname_a);
  
  // exit gracefully
  //
  return status;
}

// method: writeDataCommon
//
// arguments:
//  Sof& sof: (input) sof file object
//  const String& pname: (input) parameter name
//
// return: a boolean value indicating status
//
// this method writes data to a file for several algorithms that
// share the same parameters.
//
boolean Prediction::writeDataCommon(Sof& sof_a, const String& pname_a) const {
    
  // write the order
  //
  order_d.writeData(sof_a, PARAM_ORDER);

  // write the dynamic range
  //
  dyn_range_d.writeData(sof_a, PARAM_DYN_RANGE);
  
  // write debug level
  //
  debug_level_d.writeData(sof_a, PARAM_DBGL);

  // exit gracefully
  //
  return true;
}
