// file: $isip/class/math/scalar/Short/Short.h
// version: $Id: Short.h,v 1.31 2001/10/31 05:18:44 gao Exp $
//

// make sure definitions are only made once
//
#ifndef ISIP_SHORT
#define ISIP_SHORT

// isip include files
//
#ifndef ISIP_SCALAR
#include <MScalar.h>
#endif

// Short: a class that manages a 16-bit signed integer. this class mainly
// inherits the MScalar template functionality. we only need to add Sof file
// support and memory management
//
class Short : public MScalar<short, int16> {

  //---------------------------------------------------------------------------
  //
  // public constants
  //
  //---------------------------------------------------------------------------
public:

  // define the class name
  //
  static const String CLASS_NAME;

  //----------------------------------------
  //
  // other important constants
  //
  //----------------------------------------
    
  //----------------------------------------
  //
  // default values and arguments
  //
  //----------------------------------------  

  // define the default value(s) of the class data
  //
  
  // default arguments to methods
  //
  
  //----------------------------------------
  //
  // error codes
  //
  //----------------------------------------  

  static const long ERR = 20700;

  //---------------------------------------------------------------------------
  //
  // protected data
  //
  //---------------------------------------------------------------------------
protected:

  // static memory manager
  //
  static MemoryManager mgr_d;
  
  //---------------------------------------------------------------------------
  //
  // required public methods
  //
  //---------------------------------------------------------------------------
public:
  
  // method: name
  //
  static const String& name() {
    return CLASS_NAME;
  }

  // other static methods
  //
  static boolean diagnose(Integral::DEBUG debug_level);

  // debug methods:
  //  the setDebug method is inherited from the MScalar template class
  //
  boolean debug(const unichar* message) const;
  
  // method: destructor
  //
  ~Short() {}

  // method: default constructor
  //
  Short(short arg = DEF_VALUE) {
    value_d = arg;
  }

  // method: copy constructor
  //
  Short(const Short& arg) {
    assign(arg.value_d);
  }

  // assign methods:
  //  these methods are inherited from the MScalar template class
  //

  // method: operator=
  //
  Short& operator= (const Short& arg) {
    assign(arg);
    return *this;
  }
    
  // i/o methods:
  //  the sofSize, readData and writeData are inherited from the MScalar
  //  template class
  //

  // method: read
  //
  boolean read(Sof& sof, long tag, const String& name = CLASS_NAME) {
    return MScalar<short, int16>::read(sof, tag, name);
  }

  // method: write
  //
  boolean write(Sof& sof, long tag, const String& name = CLASS_NAME) const {
    return MScalar<short, int16>::write(sof, tag, name);
  }

  // equality methods:
  //  these methods are inherited from the MScalar template class
  //

  // method: new
  //
  static void* operator new(size_t size) {
    return mgr_d.get();
  }

  // method: new[]
  //
  static void* operator new[](size_t size) {
    return mgr_d.getBlock(size);
  }

  // method: delete
  //
  static void operator delete(void* ptr) {
    mgr_d.release(ptr);
  }

  // method: delete[]
  //
  static void operator delete[](void* ptr) {
    mgr_d.releaseBlock(ptr);
  }

  // method: setGrowSize
  //
  static boolean setGrowSize(long grow_size) {
    return mgr_d.setGrow(grow_size);
  }

  // other memory management:
  //  these methods are inherited from the MScalar template class
  //

  //---------------------------------------------------------------------------
  //
  // class-specific public methods
  //
  //---------------------------------------------------------------------------
public:
  
  // method: operator=
  //
  Short& operator= (short arg) {
    value_d = arg;
    return *this;
  }
  
  //---------------------------------------------------------------------------
  //
  // private methods
  //
  //---------------------------------------------------------------------------
private:
  //declare trigonometric methods as private because they cannot be
  //defined for Short operations
  //
  short acos(short);
  short acosh(short);
  short asin(short);
  short asinh(short);
  short atan(short);
  short atanh(short);
  short cos(short);
  short cosh(short);
  short sin(short);
  short sinh(short);
  short tan(short);
  short tanh(short);
  
  //declare those methods as private which cannot be defined for
  //integer numbers
  //  
  short ceil(short);
  short rceil(short);
  short rfloor(short);
  short round(short);
  short floor(short);

};

// end of include file
//
#endif
