// file: $isip/class/math/vector/VectorComplexDouble/VectorComplexDouble.h
// version: $Id: VectorComplexDouble.h,v 1.6 2001/10/31 19:40:53 gao Exp $
//

// make sure definitions are only made once
//
#ifndef ISIP_VECTOR_COMPLEX_DOUBLE
#define ISIP_VECTOR_COMPLEX_DOUBLE

// isip include files
//
#ifndef ISIP_MVECTOR
#include <MVector.h>
#endif

#ifndef ISIP_VECTOR_DOUBLE
#include <VectorDouble.h>
#endif

// VectorComplexDouble: this is a vector of ComplexDouble objects, which
// inherits the MVector template class.
//
class VectorComplexDouble : public MVector<ComplexDouble, complexdouble> {

  //---------------------------------------------------------------------------
  //
  // public constants
  //
  //---------------------------------------------------------------------------
public:

  // define the class name
  //
  static const String CLASS_NAME;

  //----------------------------------------
  //
  // default values and arguments
  //
  //----------------------------------------  
  
  //----------------------------------------
  //
  // error codes
  //
  //----------------------------------------  
  
  static const long ERR = 23100;

  //---------------------------------------------------------------------------
  //
  // protected data
  //
  //---------------------------------------------------------------------------
protected:

  // static memory manager
  //
  static MemoryManager mgr_d;

  //---------------------------------------------------------------------------
  //
  // required public methods
  //
  //---------------------------------------------------------------------------
public:
 
  // method: name
  //
  static const String& name() {
    return CLASS_NAME;
  }

  // other static methods
  //
  static boolean diagnose(Integral::DEBUG debug_level);

  // method: setDebug
  //  these methods are inherited from the MVector template class
  //
  
  // method: debug
  //
  boolean debug(const unichar* message) const {
    return MVector<ComplexDouble, complexdouble>::debug(name(), &message[0]);
  }
  
  // method: destructor
  //
  ~VectorComplexDouble() {}
    
  // method: default constructor
  //
  VectorComplexDouble(long length = DEF_LENGTH,
		      complexdouble value = DEF_VALUE) {
    assign(length, value);
  }

  // method: copy constructor
  //
  VectorComplexDouble(const VectorComplexDouble& vector) {
    assign(vector);
  }
  
  // assign methods:
  //  these methods are inherited from the MVector template class
  //

  // method: operator=
  //
  //  note that we need to have the overloaded operator '=' defined in the
  //  derived classes instead of the base template class. this is required
  //  since c++ predefines the operator '=' for any class by default
  //  and this hides any other definition of this operator in the base class
  //
  VectorComplexDouble& operator= (const VectorComplexDouble& vector) {
    assign(vector);
    return *this;
  }
  
  // i/o methods:
  //  the sofSize, readData and writeData are inherited from the MVector
  //  template class
  //

  // method: read
  //
  boolean read(Sof& sof, long tag, const String& name = CLASS_NAME) {
    return MVector<ComplexDouble, complexdouble>::read(sof, tag, name);
  }
  
  // method: write
  //
  boolean write(Sof& sof, long tag, const String& name = CLASS_NAME) {
    return MVector<ComplexDouble, complexdouble>::write(sof, tag, name);
  }    

  // equality methods:
  //  these methods are inherited from the MVector template class
  //

  // method: new
  //
  static void* operator new(size_t size) {
    return mgr_d.get();
  }

  // method: new[]
  //
  static void* operator new[](size_t size) {
    return mgr_d.getBlock(size);
  }

  // method: delete
  //
  static void operator delete(void* ptr) {
    mgr_d.release(ptr);
  }

  // method: delete[]
  //
  static void operator delete[](void* ptr) {
    mgr_d.releaseBlock(ptr);
  }

  // method: setGrowSize
  //
  static boolean setGrowSize(long grow_size) {
    return mgr_d.setGrow(grow_size);
  }

  // other memory management methods:
  //  the clear method is inherited from the MVector template class
  // 

  //---------------------------------------------------------------------------
  //
  // class-specific public methods
  //
  //---------------------------------------------------------------------------
public:
  
  // method: constructor
  //  this method constructs a vector with a String* object
  //
  VectorComplexDouble(const String& arg, unichar delim = DEF_DELIM) {
    assign(arg, delim);
  }
  
  // method: constructor
  //  this method constructs a vector with a unichar* object
  //
  VectorComplexDouble(const unichar* arg, unichar delim = DEF_DELIM) {
    assign(arg, delim);
  }
  
  // method: operator=
  //  this method assigns a double value to the vector
  //
  VectorComplexDouble& operator= (complexdouble value) {
    assign(value);
    return *this;
  }

  //---------------------------------------------------------------------------
  //
  // class-specific public methods: complex functionality
  //
  //---------------------------------------------------------------------------
public:

  // method: real
  //
  boolean real(VectorDouble& arg) const {
    arg.setLength(length_d);
    for (long i = (long)length_d - 1; i >= 0; i--) {
      arg(i) = v_d[i].real();
    }
    return true;
  }

  // method: imag
  //
  boolean imag(VectorDouble& arg) const {
    arg.setLength(length_d);
    for (long i = (long)length_d - 1; i >= 0; i--) {
      arg(i) = v_d[i].imag();
    }
    return true;
  }
  
  // method: abs
  //
  boolean abs(VectorDouble& arg) const {
    arg.setLength(length_d);
    for (long i = (long)length_d - 1; i >= 0; i--) {
      arg(i) = v_d[i].mag();
    }
    return true;
  }
  
  // method: mag
  //
  boolean mag(VectorDouble& arg) const {
    arg.setLength(length_d);
    for (long i = (long)length_d - 1; i >= 0; i--) {
      arg(i) = v_d[i].mag();
    }
    return true;
  }

  // method: angle
  //
  boolean angle(VectorDouble& arg) const {
    arg.setLength(length_d);
    for (long i = (long)length_d - 1; i >= 0; i--) {
      arg(i) = v_d[i].angle();
    }
    return true;
  }
  
  // method: conjugate
  //
  boolean conjugate(const VectorComplexDouble& arg) {
    setLength(arg.length_d);
    for (long i = (long)(arg.length_d) - 1; i >= 0; i--) {
      v_d[i] = arg(i).conjugate();
    }
    return true;
  }

  // method: conjugate
  //
  boolean conjugate() {
    return conjugate(*this);
  }

  //---------------------------------------------------------------------------
  //
  // private methods
  //
  //---------------------------------------------------------------------------
private:

  //declare those methods as private because they cannot be
  //defined for Complex value
  //  
  boolean operator %(const VectorComplexDouble& vector);
  boolean operator %=(const VectorComplexDouble& vector);
  boolean band();
  boolean bor();
  boolean bxor();
  boolean brs();
  boolean bls();
  boolean bcmpl();

  boolean mod();

  boolean centerClip();
  boolean fraction();
  boolean limit();
  boolean limitMag();  
  
  
};

// end of include file
//
#endif
