// file: $isip/class/math/vector/VectorUllong/VectorUllong.h
// version: $Id: VectorUllong.h,v 1.30 2001/10/31 19:43:51 gao Exp $
//

// make sure definitions are only made once
//
#ifndef ISIP_VECTOR_ULLONG
#define ISIP_VECTOR_ULLONG

// isip include files
//
#ifndef ISIP_MVECTOR
#include <MVector.h>
#endif

// VectorUllong: this is a vector of Ullong objects, which inherits
// the MVector template class.
//
class VectorUllong : public MVector<Ullong, ullong> {

  //---------------------------------------------------------------------------
  //
  // public constants
  //
  //---------------------------------------------------------------------------
public:

  // define the class name
  //
  static const String CLASS_NAME;

  //----------------------------------------
  //
  // default values and arguments
  //
  //----------------------------------------  
  
  //----------------------------------------
  //
  // error codes
  //
  //----------------------------------------  
  
  static const long ERR = 22400;

  //---------------------------------------------------------------------------
  //
  // protected data
  //
  //---------------------------------------------------------------------------
protected:

  // static memory manager
  //
  static MemoryManager mgr_d;

  //---------------------------------------------------------------------------
  //
  // required public methods
  //
  //---------------------------------------------------------------------------
public:
 
  // method: name
  //
  static const String& name() {
    return CLASS_NAME;
  }

  // other static methods
  //
  static boolean diagnose(Integral::DEBUG debug_level);

  // method: setDebug
  //  these methods are inherited from the MVector template class
  //
  
  // method: debug
  //
  boolean debug(const unichar* message) const {
    return MVector<Ullong, ullong>::debug(name(), &message[0]);
  }
  
  // method: destructor
  //
  ~VectorUllong() {}
    
  // method: default constructor
  //
  VectorUllong(long length = DEF_LENGTH, ullong value = DEF_VALUE) {
    assign(length, value);
  }

  // method: copy constructor
  //
  VectorUllong(const VectorUllong& vector) {
    assign(vector);
  }
  
  // assign methods:
  //  these methods are inherited from the MVector template class
  //

  // method: operator=
  //
  //  note that we need to have the overloaded operator '=' defined in the
  //  derived classes instead of the base template class. this is required
  //  since c++ predefines the operator '=' for any class by default
  //  and this hides any other definition of this operator in the base class
  //
  VectorUllong& operator= (const VectorUllong& vector) {
    assign(vector);
    return *this;
  }
  
  // i/o methods:
  //  the sofSize, readData and writeData are inherited from the MVector
  //  template class
  //

  // method: read
  //
  boolean read(Sof& sof, long tag, const String& name = CLASS_NAME) {
    return MVector<Ullong, ullong>::read(sof, tag, name);
  }
  
  // method: write
  //
  boolean write(Sof& sof, long tag, const String& name = CLASS_NAME) {
    return MVector<Ullong, ullong>::write(sof, tag, name);
  }    

  // equality methods:
  //  these methods are inherited from the MVector template class
  //

  // method: new
  //
  static void* operator new(size_t size) {
    return mgr_d.get();
  }

  // method: new[]
  //
  static void* operator new[](size_t size) {
    return mgr_d.getBlock(size);
  }

  // method: delete
  //
  static void operator delete(void* ptr) {
    mgr_d.release(ptr);
  }

  // method: delete[]
  //
  static void operator delete[](void* ptr) {
    mgr_d.releaseBlock(ptr);
  }

  // method: setGrowSize
  //
  static boolean setGrowSize(long grow_size) {
    return mgr_d.setGrow(grow_size);
  }

  // other memory management methods:
  //  the clear method is inherited from the MVector template class
  // 

  //---------------------------------------------------------------------------
  //
  // class-specific public methods
  //
  //---------------------------------------------------------------------------
public:
  
  // method: constructor
  //  this method constructs a vector with a String* object
  //
  VectorUllong(const String& arg, unichar delim = DEF_DELIM) {
    assign(arg, delim);
  }
  
  // method: constructor
  //  this method constructs a vector with a unichar* object
  //
  VectorUllong(const unichar* arg, unichar delim = DEF_DELIM) {
    assign(arg, delim);
  }
  
  // method: operator=
  //  this method assigns a ullong value to the vector
  //
  VectorUllong& operator= (ullong value) {
    assign(value);
    return *this;
  }

  //---------------------------------------------------------------------------
  //
  // private methods
  //
  //---------------------------------------------------------------------------
private:
  //declare trigonometric methods as private because they cannot be
  //defined for Ullong operations
  //
  ullong acos(ullong);
  ullong acosh(ullong);
  ullong asin(ullong);
  ullong asinh(ullong);
  ullong atan(ullong);
  ullong atanh(ullong);
  ullong cos(ullong);
  ullong cosh(ullong);
  ullong sin(ullong);
  ullong sinh(ullong);
  ullong tan(ullong);
  ullong tanh(ullong);
  
  //declare those methods as private which cannot be defined for
  //unsigned numbers
  //
  ullong abs(ullong);
  ullong neg(ullong);
  ullong sign(ullong);

  //declare those methods as private which cannot be defined for
  //integer numbers
  //  
  ullong ceil(ullong);
  ullong rceil(ullong);
  ullong rfloor(ullong);
  ullong round(ullong);
  ullong floor(ullong);
  
  
};

// end of include file
//
#endif
