// file: $isip/class/math/vector/VectorUlong/VectorUlong.h
// version: $Id: VectorUlong.h,v 1.31 2001/10/31 19:43:57 gao Exp $
//

// make sure definitions are only made once
//
#ifndef ISIP_VECTOR_ULONG
#define ISIP_VECTOR_ULONG

// isip include files
//
#ifndef ISIP_MVECTOR
#include <MVector.h>
#endif

// VectorUlong: this is a vector of Ulong objects, which inherits
// the MVector template class.
//
class VectorUlong : public MVector<Ulong, ulong> {

  //---------------------------------------------------------------------------
  //
  // public constants
  //
  //---------------------------------------------------------------------------
public:

  // define the class name
  //
  static const String CLASS_NAME;

  //----------------------------------------
  //
  // default values and arguments
  //
  //----------------------------------------  
  
  //----------------------------------------
  //
  // error codes
  //
  //----------------------------------------  
  
  static const long ERR = 22300;

  //---------------------------------------------------------------------------
  //
  // protected data
  //
  //---------------------------------------------------------------------------
protected:

  // static memory manager
  //
  static MemoryManager mgr_d;

  //---------------------------------------------------------------------------
  //
  // required public methods
  //
  //---------------------------------------------------------------------------
public:
 
  // method: name
  //
  static const String& name() {
    return CLASS_NAME;
  }

  // other static methods
  //
  static boolean diagnose(Integral::DEBUG debug_level);

  // method: setDebug
  //  these methods are inherited from the MVector template class
  //
  
  // method: debug
  //
  boolean debug(const unichar* message) const {
    return MVector<Ulong, ulong>::debug(name(), &message[0]);
  }
  
  // method: destructor
  //
  ~VectorUlong() {}
    
  // method: default constructor
  //
  VectorUlong(long length = DEF_LENGTH, ulong value = DEF_VALUE) {
    assign(length, value);
  }

  // method: copy constructor
  //
  VectorUlong(const VectorUlong& vector) {
    assign(vector);
  }
  
  // assign methods:
  //  these methods are inherited from the MVector template class
  //

  // method: operator=
  //
  //  note that we need to have the overloaded operator '=' defined in the
  //  derived classes instead of the base template class. this is required
  //  since c++ predefines the operator '=' for any class by default
  //  and this hides any other definition of this operator in the base class
  //
  VectorUlong& operator= (const VectorUlong& vector) {
    assign(vector);
    return *this;
  }
  
  // i/o methods:
  //  the sofSize, readData and writeData are inherited from the MVector
  //  template class
  //

  // method: read
  //
  boolean read(Sof& sof, long tag, const String& name = CLASS_NAME) {
    return MVector<Ulong, ulong>::read(sof, tag, name);
  }
  
  // method: write
  //
  boolean write(Sof& sof, long tag, const String& name = CLASS_NAME) {
    return MVector<Ulong, ulong>::write(sof, tag, name);
  }    

  // equality methods:
  //  these methods are inherited from the MVector template class
  //

  // method: new
  //
  static void* operator new(size_t size) {
    return mgr_d.get();
  }

  // method: new[]
  //
  static void* operator new[](size_t size) {
    return mgr_d.getBlock(size);
  }

  // method: delete
  //
  static void operator delete(void* ptr) {
    mgr_d.release(ptr);
  }

  // method: delete[]
  //
  static void operator delete[](void* ptr) {
    mgr_d.releaseBlock(ptr);
  }

  // method: setGrowSize
  //
  static boolean setGrowSize(long grow_size) {
    return mgr_d.setGrow(grow_size);
  }

  // other memory management methods:
  //  the clear method is inherited from the MVector template class
  // 

  //---------------------------------------------------------------------------
  //
  // class-specific public methods
  //
  //---------------------------------------------------------------------------
public:
  
  // method: constructor
  //  this method constructs a vector with a String* object
  //
  VectorUlong(const String& arg, unichar delim = DEF_DELIM) {
    assign(arg, delim);
  }
  
  // method: constructor
  //  this method constructs a vector with a unichar* object
  //
  VectorUlong(const unichar* arg, unichar delim = DEF_DELIM) {
    assign(arg, delim);
  }
  
  // method: operator=
  //  this method assigns a ulong value to the vector
  //
  VectorUlong& operator= (ulong value) {
    assign(value);
    return *this;
  }

  //---------------------------------------------------------------------------
  //
  // private methods
  //
  //---------------------------------------------------------------------------
private:
  //declare trigonometric methods as private because they cannot be
  //defined for Ulong operations
  //
  ulong acos(ulong);
  ulong acosh(ulong);
  ulong asin(ulong);
  ulong asinh(ulong);
  ulong atan(ulong);
  ulong atanh(ulong);
  ulong cos(ulong);
  ulong cosh(ulong);
  ulong sin(ulong);
  ulong sinh(ulong);
  ulong tan(ulong);
  ulong tanh(ulong);
  
  //declare those methods as private which cannot be defined for
  //unsigned numbers
  //
  ulong abs(ulong);
  ulong neg(ulong);
  ulong sign(ulong);

  //declare those methods as private which cannot be defined for
  //integer numbers
  //  
  ulong ceil(ulong);
  ulong rceil(ulong);
  ulong rfloor(ulong);
  ulong round(ulong);
  ulong floor(ulong);
  
};

// end of include file
//
#endif
