// file: $isip/class/math/vector/VectorUshort/VectorUshort.h
// version: $Id: VectorUshort.h,v 1.29 2001/10/31 19:44:04 gao Exp $
//

// make sure definitions are only made once
//
#ifndef ISIP_VECTOR_USHORT
#define ISIP_VECTOR_USHORT

// isip include files
//
#ifndef ISIP_MVECTOR
#include <MVector.h>
#endif

// VectorUshort: this is a vector of Ushort objects, which inherits
// the MVector template class.
//
class VectorUshort : public MVector<Ushort, ushort> {

  //---------------------------------------------------------------------------
  //
  // public constants
  //
  //---------------------------------------------------------------------------
public:

  // define the class name
  //
  static const String CLASS_NAME;

  //----------------------------------------
  //
  // default values and arguments
  //
  //----------------------------------------  
  
  //----------------------------------------
  //
  // error codes
  //
  //----------------------------------------  
  
  static const long ERR = 22200;

  //---------------------------------------------------------------------------
  //
  // protected data
  //
  //---------------------------------------------------------------------------
protected:

  // static memory manager
  //
  static MemoryManager mgr_d;

  //---------------------------------------------------------------------------
  //
  // required public methods
  //
  //---------------------------------------------------------------------------
public:
 
  // method: name
  //
  static const String& name() {
    return CLASS_NAME;
  }

  // other static methods
  //
  static boolean diagnose(Integral::DEBUG debug_level);

  // method: setDebug
  //  these methods are inherited from the MVector template class
  //
  
  // method: debug
  //
  boolean debug(const unichar* message) const {
    return MVector<Ushort, ushort>::debug(name(), &message[0]);
  }
  
  // method: destructor
  //
  ~VectorUshort() {}
    
  // method: default constructor
  //
  VectorUshort(long length = DEF_LENGTH, ushort value = DEF_VALUE) {
    assign(length, value);
  }

  // method: copy constructor
  //
  VectorUshort(const VectorUshort& vector) {
    assign(vector);
  }
  
  // assign methods:
  //  these methods are inherited from the MVector template class
  //

  // method: operator=
  //
  //  note that we need to have the overloaded operator '=' defined in the
  //  derived classes instead of the base template class. this is required
  //  since c++ predefines the operator '=' for any class by default
  //  and this hides any other definition of this operator in the base class
  //
  VectorUshort& operator= (const VectorUshort& vector) {
    assign(vector);
    return *this;
  }
  
  // i/o methods:
  //  the sofSize, readData and writeData are inherited from the MVector
  //  template class
  //

  // method: read
  //
  boolean read(Sof& sof, long tag, const String& name = CLASS_NAME) {
    return MVector<Ushort, ushort>::read(sof, tag, name);
  }
  
  // method: write
  //
  boolean write(Sof& sof, long tag, const String& name = CLASS_NAME) {
    return MVector<Ushort, ushort>::write(sof, tag, name);
  }    

  // equality methods:
  //  these methods are inherited from the MVector template class
  //

  // method: new
  //
  static void* operator new(size_t size) {
    return mgr_d.get();
  }

  // method: new[]
  //
  static void* operator new[](size_t size) {
    return mgr_d.getBlock(size);
  }

  // method: delete
  //
  static void operator delete(void* ptr) {
    mgr_d.release(ptr);
  }

  // method: delete[]
  //
  static void operator delete[](void* ptr) {
    mgr_d.releaseBlock(ptr);
  }

  // method: setGrowSize
  //
  static boolean setGrowSize(long grow_size) {
    return mgr_d.setGrow(grow_size);
  }

  // other memory management methods:
  //  the clear method is inherited from the MVector template class
  // 

  //---------------------------------------------------------------------------
  //
  // class-specific public methods
  //
  //---------------------------------------------------------------------------
public:
  
  // method: constructor
  //  this method constructs a vector with a String* object
  //
  VectorUshort(const String& arg, unichar delim = DEF_DELIM) {
    assign(arg, delim);
  }
  
  // method: constructor
  //  this method constructs a vector with a unichar* object
  //
  VectorUshort(const unichar* arg, unichar delim = DEF_DELIM) {
    assign(arg, delim);
  }
  
  // method: operator=
  //  this method assigns a ushort value to the vector
  //
  VectorUshort& operator= (ushort value) {
    assign(value);
    return *this;
  }

  //---------------------------------------------------------------------------
  //
  // private methods
  //
  //---------------------------------------------------------------------------
private:
  
  //declare trigonometric methods as private because they cannot be
  //defined for Ushort
  //

  ushort acos(ushort);
  ushort acosh(ushort);
  ushort asin(ushort);
  ushort asinh(ushort);
  ushort atan(ushort);
  ushort atanh(ushort);
  ushort cos(ushort);
  ushort cosh(ushort);
  ushort sin(ushort);
  ushort sinh(ushort);
  ushort tan(ushort);
  ushort tanh(ushort);
  
  //declare those methods as private which cannot be defined for
  //unsigned numbers
  //
  ushort abs(ushort);
  ushort neg(ushort);
  ushort sign(ushort);

  //declare those methods as private which cannot be defined for
  //integer numbers
  //  
  ushort ceil(ushort);
  ushort rceil(ushort);
  ushort rfloor(ushort);
  ushort round(ushort);
  ushort floor(ushort); 
  
};

// end of include file
//
#endif
