// file: $isip/class/asr/AudioDatabase/audb_04.cc
// version: $Id: audb_04.cc,v 1.4 2002/08/13 14:18:55 zheng Exp $
//

// isip include files
//
#include "AudioDatabase.h"

// method: sofSize
//
// arguments: none
//
// return: logical error status
//
// this method determines the size of the object
//
long AudioDatabase::sofSize() const {

  // declare local variables
  //
  long size = 0;

  // get the size of the object
  //
  size = name_d.sofSize();
  size += hash_d.sofSize();

  // return the size
  //
  return size;
}

// method: write
//
// arguments:
//  Sof& sof: (input) sof file object
//  long tag: (input) sof object instance tag
//  const String& name: (input) sof object instance name
//
// return: logical error status
//
// this method has the object read itself from an Sof file
//
boolean AudioDatabase::write(Sof& sof_a, long tag_a,
			     const String& name_a) const {

  long obj_size;

  if (sof_a.isText()) {
    obj_size = Sof::ANY_SIZE;
  }
  else {
    obj_size = sofSize();
  }
  
  // put the object into the sof file's index
  //
  if (!sof_a.put(name_a, tag_a, obj_size)) {
    return false;
  }

  // exit gracefully
  //
  return writeData(sof_a);
}

// method: writeData
//
// arguments:
//  Sof& sof: (input) sof file object
//  const String& pname: (input) parameter name
//
// return: logical error status
//
// this method writes the object to the Sof file. it assumes that the
// Sof file is already positioned correctly.
//
boolean AudioDatabase::writeData(Sof& sof_a, const String& pname_a) const {

  // write a start string if necessary
  //
  sof_a.writeLabelPrefix(pname_a);

  // write the database name
  //
  name_d.writeData(sof_a, PARAM_NAME);
  
  // get the keys associated with the hash table
  //
  Vector<String> keys;
  if (!hash_d.keys(keys)) {
    return Error::handle(name(), L"readData", Error::ARG, __FILE__, __LINE__);
  }

  // write the keys associated with the hash table
  //
  keys.writeData(sof_a, PARAM_KEYS);

  // get the values associated with the hash table
  //
  Vector<Long> values;
  if (!hash_d.values(values)) {
    return Error::handle(name(), L"readData", Error::ARG, __FILE__, __LINE__);
  }

  // write the values associated with the hash table
  //
  values.writeData(sof_a, PARAM_VALUES);
  
  // put an end string if necessary
  //
  sof_a.writeLabelSuffix(pname_a);
  
  // exit gracefully
  //
  return true;
}
