// file: $isip/class/asr/Machine/mach_04.cc
//

// isip include files
//
#include "Machine.h"

// method: write
//
// arguments:
//  Sof& sof: (input) sof file object
//  long tag: (input) sof object instance tag
//  const String& name: (input) sof object instance name
//
// return: boolean value indicating status
//
// this method has the object write itself to an Sof file
//
boolean Machine::write(Sof& sof_a, long tag_a,
		       const String& name_a) const {

  long obj_size = 0;
  
  // write the instance of the object into the Sof file
  //
  if (sof_a.isText()) {

    // set the size to by dynamic
    //
    obj_size = Sof::ANY_SIZE;
  }
  else {

    // the size index and the size of each element
    //
    obj_size = sofSize();
  }

  // write the object into the sof file's index
  //
  if (!sof_a.put(name_a, tag_a, obj_size)) {
    return false;
  }
  
  // exit gracefully
  //
  return writeData(sof_a);
}

// method: writeData
//
// arguments:
//  Sof& sof: (input) sof file object
//  const String& pname: (input) parameter name
//
// return: boolean value indicating status
//
// this method has the object write itself to an Sof file. it assumes
// that the Sof file is already positioned correctly.
//
boolean Machine::writeData(Sof& sof_a, const String& pname_a) const {
  
  // write a start string if necessary
  //
  sof_a.writeLabelPrefix(pname_a);

  // write the protocol
  //
  PROTOCOL_MAP.writeElementData(sof_a, PARAM_PROTOCOL, (long)protocol_d);

  // write the node name
  //
  node_name_d.writeData(sof_a, PARAM_NODE_NAME);

  // write the architecture
  //
  architecture_d.writeData(sof_a, PARAM_ARCHITECTURE);
      
  // write the number of processors
  //
  num_processors_d.writeData(sof_a, PARAM_NUM_PROCESSORS);

  // write the speed
  //
  speed_d.writeData(sof_a, PARAM_PROCESSOR_SPEED);

  // write the main memory
  //
  main_memory_d.writeData(sof_a, PARAM_MAIN_MEMORY);

  // write the swap memory
  //
  swap_memory_d.writeData(sof_a, PARAM_SWAP_MEMORY);

  // write the OS
  //
  os_d.writeData(sof_a, PARAM_OS_NAME);

  // write the OS version
  //
  os_version_d.writeData(sof_a, PARAM_OS_VERSION);
  
  // exit gracefully
  //
  return true;
}
