// file: $isip/class/pr/HiddenMarkovModel/hmm_04.cc
//

// isip include files
//
#include "HiddenMarkovModel.h"
#include <Sof.h>

// method: write
//
// arguments:
//  Sof& sof: (input) sof file object
//  long tag: (input) sof object instance tag
//  const String& name: (input) sof object instance name
//
// return: logical error status
//
// this method has the object read itself from an Sof file
//
boolean HiddenMarkovModel::write(Sof& sof_a, long tag_a,
			     const String& name_a) const {

  long obj_size;

  if (sof_a.isText()) {
    obj_size = Sof::ANY_SIZE;
  }
  else {
    obj_size = sofSize();
  }
  
  // put the object into the sof file's index
  //
  if (!sof_a.put(name_a, tag_a, obj_size)) {
    return false;
  }

  // exit gracefully
  //
  return writeData(sof_a);
}

// method: writeData
//
// arguments:
//  Sof& sof: (input) sof file object
//  const String& pname: (input) parameter name
//
// return: logical error status
//
// this method writes the object to the Sof file. it assumes that the
// Sof file is already positioned correctly.
//
boolean HiddenMarkovModel::writeData(Sof& sof_a, const String& pname_a) const {
  
  // write a start string if necessary
  //
  sof_a.writeLabelPrefix(pname_a);

  // write the algorithm
  //
  ALGO_MAP.writeElementData(sof_a, PARAM_ALGORITHM, (long)algorithm_d);

  // write the implementation
  //
  IMPL_MAP.writeElementData(sof_a, PARAM_IMPLEMENTATION, (long)implementation_d);

  // put an end string if necessary
  //
  sof_a.writeLabelSuffix(pname_a);
  
  // exit gracefully
  //
  return true;
}

// method: sofSize
//
// arguments: none
//
// return: size of object
//
// this method returns the size of the object in the Sof file and is
// used for binary write
//
long HiddenMarkovModel::sofSize() const {

  // count up object by object
  //
  long size = 0;

  // accumulate the sofSize of all the data being written
  //
  size += ALGO_MAP.elementSofSize();

  size += IMPL_MAP.elementSofSize();

  // return the size
  //
  return size;
}
