// file: $isip/class/search/ContextMap/cmap_03.cc
// version: $Id: cmap_03.cc,v 1.5 2002/12/13 22:25:56 parihar Exp $
//

// isip include files
//
#include "ContextMap.h"
#include  <JSGFParser.h>

// method: read
//
// arguments:
//  Sof& sof: (input) sof file object
//  long tag: (input) sof object instance tag
//  const String& name: (input) sof object instance name
//
// return: a boolean value indicating status
//
// this method has the object read itself from an Sof file
//
boolean ContextMap::read(Sof& sof_a, long tag_a, const String& name_a) {

  // read the instance of the object from the Sof file
  //
  if (!sof_a.find(name_a, tag_a)) {
    return false;
  }
  
  // read the actual data from the sof file
  //
  if (!readData(sof_a)) {
    return false;
  }
  
  // exit gracefully
  //
  return true;  
}

// method: readData
//
// arguments:
//  Sof& sof: (input) sof file object
//  const String& pname: (input) parameter name
//  long size: (input) size in bytes of object (or full_size)
//  boolean param: (input) is the parameter name in the file?
//  boolean nested: (input) are we nested?
//
// return: a boolean value indicating status
//
// this method has the object read itself from an Sof file. it assumes
// that the Sof file is already positioned correctly.
//
boolean ContextMap::readData(Sof& sof_a, const String& pname_a,
			      long size_a, boolean param_a,
			      boolean nested_a) {

  Integral::DEBUG debug_level = Integral::NONE;
  SofParser parser;
  parser.setDebug(debug_level);
    
  // are we nested?
  //
  if (nested_a) {
    parser.setNest();
  }
  
  // load the parser
  //
  if (!parser.load(sof_a, size_a)) {
    return Error::handle(name(), L"readData", Error::READ,
			 __FILE__, __LINE__, Error::WARNING);
  }	    

  // read the context
  //
  if (!context_d.readData(sof_a, PARAM_CONTEXT,
			  parser.getEntry(sof_a, PARAM_CONTEXT),
			  false, false)) {
    return Error::handle(name(), L"readData", Error::READ, __FILE__, __LINE__, Error::WARNING);
  }

  // read the index
  //
  if (!index_d.readData(sof_a, PARAM_CONTEXT_INDEX,
			      parser.getEntry(sof_a, PARAM_CONTEXT_INDEX))) {
    return Error::handle(name(), L"readData", Error::READ, __FILE__, __LINE__, Error::WARNING);
  }      
  
  // exit gracefully
  //
  return true;
}
