// file: $isip/class/sp/FrontEnd/fe_03.cc
// version: $Id: fe_03.cc,v 1.3 2002/07/14 15:39:01 picone Exp $
//

// isip include files
//
#include "FrontEnd.h"

// method: read
//
// arguments:
//  Sof& sof: (input) sof file object
//  long tag: (input) sof object instance tag
//  const String& name: (input) sof object instance name
//
// return: a boolean value indicating status
//
// this method has the object read itself from an Sof file
//
boolean FrontEnd::read(Sof& sof_a, long tag_a, const String& name_a) {

  // read the instance of the object from the Sof file
  //
  if (!sof_a.find(name_a, tag_a)) {
    return false;
  }
  
  // read the actual data from the sof file
  //
  if (!readData(sof_a)) {
    return false;
  }
  
  // exit gracefully
  //
  return true;  
}

// method: readData
//
// arguments:
//  Sof& sof: (input) sof file object
//  const String& pname: (input) parameter name
//  long size: (input) size in bytes of object (or full_size)
//  boolean param: (input) is the parameter name in the file?
//  boolean nested: (input) are we nested?
//
// return: a boolean value indicating status
//
// this method has the object read itself from an Sof file. it assumes
// that the Sof file is already positioned correctly.
//
boolean FrontEnd::readData(Sof& sof_a, const String& pname_a,
			   long size_a, boolean param_a,
			   boolean nested_a) {

  // set the parser
  //
  SofParser* parser = new SofParser;
  parser->setDebug(debug_level_d);
  
  // ignore implicit parameter setting
  //
  
  // are we nested?
  //
  if (nested_a) {
    parser->setNest();
  }

  long pos = sof_a.tell();
  
  // load the parse
  //
  if (!parser->load(sof_a, size_a)) {
    return Error::handle(name(), L"readData", Error::READ,
			 __FILE__, __LINE__, Error::WARNING);
  }
   
  // read the type
  //
  TYPES type;
  if (!TYPES_MAP.readElementData((long&)type, sof_a, PARAM_TYPES,
				 parser->getEntry(sof_a, PARAM_TYPES))) {
    reset();

    if (parser != (SofParser*)NULL) {
      delete parser;
      parser = (SofParser*)NULL;
    }
    
    return Error::handle(name(), L"readData", Error::READ,
                         __FILE__, __LINE__, Error::WARNING);
  }      

  // set the type
  //
  reset();
  setType(type);

  if (sof_a.isText()) {
    sof_a.seek(pos, File::POS);
  }
  
  // pass this parser to the sub-read method. the sub-method will
  // delete the object.
  //
  virtual_fe_d->setParser(parser);

  // read the data
  //
  if (!virtual_fe_d->readData(sof_a, pname_a, size_a, param_a, nested_a)) {
    return Error::handle(name(), L"readData", Error::READ,
			 __FILE__, __LINE__, Error::WARNING);
  }

  // exit gracefully
  //
  return true;
}
