// file: $isip/class/algo/Recipe/rcp_03.cc
// version: $Id: rcp_03.cc,v 1.9 2002/07/13 04:37:29 picone Exp $
//

// isip include files
//
#include "Recipe.h"

// method: read
//
// arguments:
//  Sof& sof: (input) sof file object
//  long tag: (input) sof object instance tag
//  const String& name: (input) sof object instance name
//
// return: a boolean value indicating status
//
// this method reads an object from an Sof file according
// to the specified name and tag.
//
boolean Recipe::read(Sof& sof_a, long tag_a, const String& name_a) {

  // read the instance of the object from the Sof file
  //
  if (!sof_a.find(name_a, tag_a)) {
    return false;
  }
  
  // read the actual data from the sof file
  //
  return readData(sof_a);
}


// method: readData
//
// arguments:
//  Sof& sof: (input) sof file object
//  const String& pname: (input) parameter name
//  long size: (input) size in bytes of object (or FULL_OBJECT)
//  boolean param: (input) is the parameter name in the file?
//  boolean nested: (input) are we nested?
//
// return: a boolean value indicating status
//
// this method reads an object from an Sof file. it assumes
// that the Sof file is already positioned correctly.
//
boolean Recipe::readData(Sof& sof_a, const String& pname_a, long size_a,
			     boolean param_a, boolean nested_a) {

  // declare local variable
  //
  SofParser parser;

  // set the debug level of the parser
  //
  parser.setDebug(debug_level_d);
  
  // are we nested?
  //
  if (nested_a) {
    parser.setNest();
  }
    
  // load the parse
  //
  if (!parser.load(sof_a, size_a)) {

    // return a warning message
    //
    return Error::handle(name(), L"readData", Error::READ,
			 __FILE__, __LINE__, Error::WARNING);
  }          

  // get the digraph
  //
  if (parser.isPresent(sof_a, PARAM_RECIPE)) {
    if (!recipe_d.readData(sof_a, PARAM_RECIPE,
				parser.getEntry(sof_a, PARAM_RECIPE),
				false, true)) {

      // return a warning message
      //
      return Error::handle(name(), L"readData", Error::READ,
			   __FILE__, __LINE__, Error::WARNING);
    }      
  }
  else {
    recipe_d.clear();
  }

  // exit gracefully
  //
  return true;
}
