// file: $isip/class/stat/Histogram/hist_04.cc
// version: $Id: hist_04.cc,v 1.1 2001/02/01 16:47:50 hamaker Exp $
//

// isip include files
//
#include "Histogram.h"

// method: write
//
// arguments:
//  Sof& sof: (input) sof file object
//  long tag: (input) sof object instance tag
//  const String& name: (input) sof object instance name
//
// return: boolean value indicating status
//
// this method has the object write itself to an Sof file
//
boolean Histogram::write(Sof& sof_a, long tag_a,
			 const String& name_a) const {

  long obj_size = 0;
  
  // write the instance of the object into the Sof file
  //
  if (sof_a.isText()) {

    // set the size to by dynamic
    //
    obj_size = Sof::ANY_SIZE;
  }
  else {

    // the size index and the size of each element
    //
    obj_size = sofSize();
  }

  // write the object into the sof file's index
  //
  if (!sof_a.put(name_a, tag_a, obj_size)) {
    return false;
  }
  
  // exit gracefully
  //
  return writeData(sof_a);
}

// method: writeData
//
// arguments:
//  Sof& sof: (input) sof file object
//  const String& pname: (input) parameter name
//
// return: boolean value indicating status
//
// this method has the object write itself to an Sof file. it assumes
// that the Sof file is already positioned correctly.
//
boolean Histogram::writeData(Sof& sof_a, const String& pname_a) const {
  
  // write a start string if necessary
  //
  sof_a.writeLabelPrefix(pname_a);

  // write the bin mode
  //
  MODE_MAP.writeElementData(sof_a, PARAM_MODE, (long)mode_d);
  
  // write the vector of bins
  //
  bins_d.writeData(sof_a, PARAM_BINS);

  // write the counts
  //
  counts_d.writeData(sof_a, PARAM_COUNTS);

  // put an end string if necessary
  //
  sof_a.writeLabelSuffix(pname_a);
  
  // exit gracefully
  //
  return true;
}
