// file: $isip/class/stat/NGramModel/ngrm_03.cc
// version: $Id: ngrm_03.cc,v 1.1 2002/07/30 05:03:25 zheng Exp $
//

// isip include files
//
#include "NGramModel.h"
#include <Sof.h>

// method: read
//
// arguments:
//  Sof& sof: (input) sof file object
//  const long tag: (input) sof object instance tag
//  const String& name: (input) sof object instance name
//
// return: a boolean value indicating status
//
// this method has the object read itself from an Sof file
//
boolean NGramModel::read(Sof& sof_a, const long tag_a, const String& name_a) {

  // get the instance of the object from the Sof file
  //
  if (!sof_a.find(name_a, tag_a)) {
    return false;
  }

  // read the actual data from the sof file
  //
  if (!readData(sof_a)) {
    return false;
  }

  // exit gracefully
  //
  return true;
}

// method: readData
//
// arguments:
//  Sof& sof: (input) sof file object
//  const String& pname: (input) parameter name
//  long size: (input) number of bytes in file
//  boolean param: (input) is the parameter specified?
//  boolean nested: (input) is this nested?
//
// return: a boolean value indicating status
//
// this method has the object read itself from an Sof file. it assumes
// that the Sof file is already positioned correctly.
//
boolean NGramModel::readData(Sof& sof_a, const String& pname_a, long size_a,
			     boolean param_a, boolean nested_a) {

  // if ascii, parse the start bracket
  //
  SofParser parser;
    
  nested_a = true;
  
  if (nested_a) {
    parser.setNest();
  }

  // set the debug level
  //
  parser.setDebug(debug_level_d);
  
  if (nested_a) {
    parser.setNest();
  }

  // load the parse
  //
  if (!parser.load(sof_a, size_a)) {

    // return a warning message
    //
    return Error::handle(name(), L"readData", Error::READ,
			 __FILE__, __LINE__, Error::WARNING);
  }           

  // read the data
  //
  if (parser.isPresent(sof_a, PARAM_ORDER)) {
    if (!order_d.readData(sof_a, PARAM_ORDER,
			  parser.getEntry(sof_a, PARAM_ORDER))) {
      return Error::handle(name(), L"readData", Error::READ,
			   __FILE__, __LINE__, Error::WARNING);
    }	      
  }
  else {
    order_d.assign(DEF_ORDER);
  }

  // read the keys associated with the hash table
  //
  Vector<Long> keys;
  if (parser.isPresent(sof_a, PARAM_KEYS)) {
    if (!keys.readData(sof_a, PARAM_KEYS,
			   parser.getEntry(sof_a, PARAM_KEYS),
			   false, false)) {
      
	// return a warning message
	//
	return Error::handle(name(), L"readData", Error::READ,
			     __FILE__, __LINE__, Error::WARNING);
      }
  }

  // read the values associated with the hash table
  //
  Vector<NGramNode> values;
  if (parser.isPresent(sof_a, PARAM_VALUES)) {
    if (!values.readData(sof_a, PARAM_VALUES,
			 parser.getEntry(sof_a, PARAM_VALUES),
			 false, false)) {
      
      // return a warning message
      //
      return Error::handle(name(), L"readData", Error::READ,
			   __FILE__, __LINE__, Error::WARNING);
      }
  }

  // write hash table
  //
  gram_hash_d.clear(Integral::RESET);
  if (keys.length() > 0 && values.length() > 0) {
    long len = keys.length();
    for (long i=0; i < len; i++) {
      gram_hash_d.insert(keys(i), &values(i));
    }
  }
  
  // make sure all coefficients have been accessed
  //
  if (!parser.checkParams(sof_a)) {
    return Error::handle(name(), L"readData", Error::IO, __FILE__, __LINE__,
			 Error::WARNING);
  }
  
  // exit gracefully
  //
  return true;
}
