// file: $isip/class/stat/UniformModel/uni_04.cc
// version: $Id: uni_04.cc,v 1.3 2001/04/24 15:28:14 peng Exp $
//

// isip include files
//
#include "UniformModel.h"

// method: write
//
// arguments:
//  Sof& sof: (input) sof file object
//  long tag: (input) sof object instance tag
//  const String& name: (input) sof object instance name
//
// return: logical error status
//
// this method has the object write itself to an Sof file
//
boolean UniformModel::write(Sof& sof_a, long tag_a,
			    const String& name_a) const {

  // write the instance of the object into the Sof file
  //
  long obj_size = 0;

  if (sof_a.isText()) {

    // set the size to by dynamic
    //
    obj_size = Sof::ANY_SIZE;
  }

  else {

    // the size index and the size of each element
    //
    obj_size = sofSize();
  }

  // write the object into the sof file's index
  //
  if (!sof_a.put(name_a, tag_a, obj_size)) {
    return false;
  }
  
  // exit gracefully
  //
  return writeData(sof_a);
}

// method: writeData
//
// arguments:
//  Sof& sof: (input) sof file object
//  const String& pname: (input) parameter name
//
// return: logical error status
//
// this method has the object writes itself from an Sof file. it assumes
// that the Sof file is already positioned correctly.
//
boolean UniformModel::writeData(Sof& sof_a, const String& pname_a) const {
  
  // write a start string if necessary
  //
  sof_a.writeLabelPrefix(pname_a);

  // write the min's
  //
  min_d.writeData(sof_a, PARAM_MIN);
  
  // write the max's
  //
  max_d.writeData(sof_a, PARAM_MAX);
  
  // put an end string if necessary
  //
  sof_a.writeLabelSuffix(pname_a);
  
  // exit gracefully
  //
  return true;
}
