;; file: $isip/login/editor_emacs_modes.el
;;
;; this file sets various variables and modes
;;

;;---------------------------------------------------------------------------;;
;;									     ;;
;; html related settings					     	     ;;
;;									     ;;
;;---------------------------------------------------------------------------;;
(autoload 'html-helper-mode "html-helper-mode" "HTML major mode." t)
(defvar html-helper-address-string (concat "www@" (getenv "ISIP_DOMAIN"))
  "*The default author string of each file.")

;;---------------------------------------------------------------------------;;
;;									     ;;
;; netscape related settings					     	     ;;
;;									     ;;
;;---------------------------------------------------------------------------;;
(autoload 'browse-url-at-point "browse-url"
  "Ask a WWW browser to load the URL at or before point." t)
(autoload 'browse-url-at-mouse "browse-url"
  "Ask a WWW browser to load a URL clicked with the mouse." t)
(autoload 'browse-url-of-buffer "browse-url"
  "Ask a WWW browser to display BUFFER." t)
(autoload 'browse-url-of-file "browse-url"
  "Ask a WWW browser to display FILE." t)
(autoload 'browse-url-of-dired-file "browse-url"
  "In Dired, ask a WWW browser to display the file named on this line." t)

;;---------------------------------------------------------------------------;;
;;									     ;;
;; gnus/netnews related settings					     ;;
;;									     ;;
;;---------------------------------------------------------------------------;;

;; setup the nntp server
;;
(if (and (not (boundp 'gnus-nntp-server)) (file-directory-p "/usr/spool/news"))
    (progn (setq gnus-nntp-service nil))
  (setq gnus-nntp-server (getenv "ISIP_SERVER_INFO")))

;; set some important fields for posting messages
;;
(setq gnus-local-domain (getenv "ISIP_DOMAIN"))
(setq gnus-local-organization (getenv "ISIP_ORGANIZATION"))
(setq gnus-article-save-directory (getenv "ISIP_MAILDIR"))
(setq gnus-author-copy (getenv "ISIP_MAILCOPY"))

;; set some modes to make gnus behave itself
;;
(setq gnus-inhibit-startup-message t)
(setq gnus-visual nil)
(setq gnus-check-new-newsgroups nil)

;; create a new function that outputs messages to an rmail file
;;
(setq gnus-startup-hook
      '(lambda ()
	 (setq gnus-save-newsrc-file nil)
	 (defun gnus-plain-save-name (newsgroup headers &optional last-file)
	   (concat (getenv "ISIP_MAILDIR") (getenv "ISIP_GNUSFOLDER")))))

;; define appropriate key bindings for reply-with-yank and 
;; followup-with-yank
;;
(setq gnus-summary-mode-hook
      '(lambda ()
	 (define-key gnus-summary-mode-map "r"
	   'gnus-summary-reply-with-original)
	 (define-key gnus-summary-mode-map "R"
	   'gnus-summary-reply)
	 (define-key gnus-summary-mode-map "f"
	   'gnus-summary-followup-with-original)
	 (define-key gnus-summary-mode-map "F"
	   'gnus-summary-followup)))

;; define key bindings to imitate the mail environment for including
;; signature files in post followups. In reply mode the mail settings
;; are used automatically.
;;
(setq message-mode-hook
      '(lambda ()
         (define-key message-mode-map "\C-c0"
           'mail-signature-0)
         (define-key message-mode-map "\C-c1"
           'mail-signature-1)
         (define-key message-mode-map "\C-c2"
           'mail-signature-2)
         (define-key message-mode-map "\C-c3"
           'mail-signature-3)
         (define-key message-mode-map "\C-c4"
           'mail-signature-4)
         (define-key message-mode-map "\C-c5"
           'mail-signature-5)))

;; add the Reply-to and FCC fields to the message being composed
;; to be posted either as a new article, a followup or a reply
;;
(setq message-archive-file-name (getenv "ISIP_MAILCOPY"))
(setq message-default-reply-to (concat (user-login-name) "@"
				    (getenv "ISIP_DOMAIN")))
(setq message-setup-hook 
      '(lambda ()
	 (expand-abbrev)
	 (interactive)
	 (message-position-on-field "Subject")
	 (insert "\nReply-to: "message-default-reply-to)
	 (message-position-on-field "Reply-To")
	 (insert "\nFCC: " message-archive-file-name)))

;;---------------------------------------------------------------------------;;
;;									     ;;
;; mime mail tools
;;									     ;;
;;---------------------------------------------------------------------------;;

(autoload 'mime-mode "mime" "Minor mode for editing MIME message." t)

;; In case of Mail mode (includes VM mode), you need the following
;; hook definition:
(setq mail-mode-hook
      (list
       (function
        (lambda ()
          (mime-mode)))))

;; In case of News mode, you need the following hook definition:
;;
(setq news-reply-mode-hook
      (list
       (function
        (lambda ()
          (mime-mode)))))

;; Followings are for message forwarding as content-type
;; "message/rfc822".
;;
(setq rmail-mode-hook
      (list
       (function
        (lambda ()
          ;; Forward mail using MIME.
          (require 'mime)
          (substitute-key-definition 'rmail-forward
                                     'mime-forward-from-rmail-using-mail
                                     (current-local-map))
          ))))

(setq gnus-mail-forward-method 'mime-forward-from-gnus-using-mail)
(setq gnus-summary-mode-hook
      (list
       (function
	(lambda ()
	  ;; Forward article using MIME.
	  (require 'mime)
	  ))))

;; In case of Emacs 19, it is possible to emphasize the message tags
;; using font-lock mode as follows:
;;
(setq mime-mode-hook
      (list
       (function
        (lambda ()
          (font-lock-mode 1)
          (setq font-lock-keywords (list mime-tag-regexp))))))

;; First of all, define the following autoload entries:
;;
(autoload 'rmail-show-mime		"rmailmime" "Show MIME messages."  t)
(autoload 'rmail-convert-mime-header	"rmailmime" "Convert MIME header." nil)

;; To convert MIME headers into readable form automatically in Rmail,
;; set the variable rmail-message-filter to the function
;; rmail-convert-mime-header as follows:
;; 
(setq rmail-message-filter 'rmail-convert-mime-header)

;;---------------------------------------------------------------------------;;
;;									     ;;
;; mail related settings					     	     ;;
;;									     ;;
;;---------------------------------------------------------------------------;;

;; define default mail location and prompts
;;  note: implicit in this code is a standard organization scheme
;;
(setq rmail-file-name (getenv "ISIP_MAILFOLDER"))
(setq rmail-primary-inbox-list '(getenv "ISIP_MAILDIR"))
(setq rmail-last-file (getenv "ISIP_MAILDIR"))
(setq rmail-last-rmail-file rmail-last-file)
(setq rmail-default-rmail-file rmail-last-file)
;;(setq mail-self-blind t)
(setq mail-archive-file-name (getenv "ISIP_MAILCOPY"))
(setq mail-default-reply-to (concat (user-login-name) "@"
				    (getenv "ISIP_DOMAIN")))
(setq rmail-highlighted-headers nil)
;;(setq rmail-delete-after-output t)

;; define smtp variables
;;
(setq smtpmail-local-domain (getenv "ISIP_DOMAIN"))
(setq smtpmail-default-smtp-server (concat (getenv "ISIP_SERVER_MAIL") "@"
					   (getenv "ISIP_DOMAIN")))
(setq mail-host-address (getenv "ISIP_DOMAIN"))
(setq user-mail-address (concat (user-login-name) "@" (getenv "ISIP_DOMAIN")))

;; set bindings for special mail functions:
;;  (1) a binding for sending mail with aliases and exiting;
;;  (2) a binding for sending mail with aliases;
;;  (3) bindings to add different types of signatures;
;;
(setq mail-mode-hook
      '(lambda ()
	 (define-key mail-mode-map "\C-c\C-c"
	   'mail-send-and-exit-with-aliases)
	 (define-key mail-mode-map "\C-c\C-s"
	   'mail-send-with-aliases)
	 (define-key mail-mode-map "\C-c0"
	   'mail-signature-0)
	 (define-key mail-mode-map "\C-c1"
	   'mail-signature-1)
	 (define-key mail-mode-map "\C-c2"
	   'mail-signature-2)
	 (define-key mail-mode-map "\C-c3"
	   'mail-signature-3)
	 (define-key mail-mode-map "\C-c4"
	   'mail-signature-4)
	 (define-key mail-mode-map "\C-c5"
	   'mail-signature-5)
	 (define-key mail-mode-map "\C-ca"
	   'mail-approve-message)))

;; set bindings for special rmail functions:
;;  (1) save and quit;
;;  (2) reply with a yank.
;;  (3) our special output function.
;;
(setq rmail-mode-hook
      '(lambda ()
	 (define-key rmail-mode-map "q"
	   'rmail-save-quit-kill-folder)
	 (define-key rmail-mode-map "r"
	   'rmail-reply-with-yank)
	 (define-key rmail-mode-map "o"
	   'rmail-isip-output)
	 (define-key rmail-mode-map "!"
	   'rmail-show-mime)))



;;---------------------------------------------------------------------------;;
;;									     ;;
;; assist in moderation of mailing lists
;;									     ;;
;;---------------------------------------------------------------------------;;

(setq moderator-password-file 
      (substitute-in-file-name "$HOME/login/moderator_password.text"))

(defun mail-approve-message ()
  "insert the approved password for the given message"
  (interactive)
  (save-excursion
    (beginning-of-buffer)
    (if (search-forward "--text follows this line--" (point-max) t)
	
	;; then
	(progn
	(next-line 1)
	(beginning-of-line)
	(insert-file moderator-password-file)))))

;;---------------------------------------------------------------------------;;
;;									     ;;
;; backup/journal/auto-save related settings				     ;;
;;									     ;;
;;---------------------------------------------------------------------------;;

;; turn off backup/copy modes
;;
(setq make-backup-files nil)
;;(setq backup-by-copying nil)
;;(setq backup-by-copying-when-linked t)
;;(setq backup-by-copying-when-mismatch t)

;; turn off auto-save modes
;;
(setq auto-save-interval 0)
(setq auto-save-default nil)

;; turn on auto-save/backups if requested
;;
(if (and
     (getenv "ISIP_EDITOR_BACKUP_INTERVAL")
     (> (string-to-int (getenv "ISIP_EDITOR_BACKUP_INTERVAL")) 0))
    (progn
      (setq make-backup-files t)
      (setq backup-by-copying nil)
      (setq auto-save-default t)
      (setq auto-save-interval
	    (string-to-int (getenv "ISIP_EDITOR_BACKUP_INTERVAL")))
      (setq delete-auto-save-files t)))

;;---------------------------------------------------------------------------;;
;;									     ;;
;; misc. tools requiring setup						     ;;
;;									     ;;
;;---------------------------------------------------------------------------;;

;; calculator
;;
(autoload 'calc "calc" "calculator mode" t nil)
(autoload 'calc-extensions "calc-ext" nil nil nil)
(autoload 'quick-calc "calc" "quick calculator" t nil)
(autoload 'calc-grab-region "calc-ext" nil t nil)
(autoload 'defmath "calc-ext" nil t t)

(require 'iswitchb)
(iswitchb-default-keybindings)

;;---------------------------------------------------------------------------;;
;;									     ;;
;; misc. modes and interactive settings					     ;;
;;									     ;;
;;---------------------------------------------------------------------------;;

;; turn off the annoying suggest key bindings mode
;;
(setq suggest-key-bindings nil)

;; set default mode for new buffers
;;
(setq default-major-mode 'fundamental-mode)

;; turn off message logging
;;
(setq message-log-max 0)

;; activate linenumber mode
;;
(setq line-number-mode t)

;; prompt for terminating files with linefeed
;;
(setq require-final-newline 'ask)

;; set the default printer command
;;
(setq lpr-command "lp")
(setq lpr-switches '("-o nobanner" "-m"))

;; case:
;;  these statements make the downcase/upcase commands less intelligent
;;  by forcing them to ignore case. this is a compromise between the needs
;;  of programmers and text processors.
;;
(put 'downcase-region 'disabled nil)
(put 'upcase-region 'disabled nil)

;;---------------------------------------------------------------------------;;
;;									     ;;
;; c++ mode customization    					             ;;
;;									     ;;
;;---------------------------------------------------------------------------;;
(add-hook 'c++-mode-hook
	  '(lambda () (local-set-key "\r" 'newline-and-indent)))
(add-hook 'c++-mode-hook
	  '(lambda () (local-set-key [C-tab] 'tab-to-tab-stop)))
(add-hook 'c++-mode-hook
	  '(lambda () (local-set-key "\C-x!" 'isip-c++-insert-error-handler)))

;; add the extra types to c++ mode for font-lock
;;
(setq c++-font-lock-extra-types 
      '("boolean" "llong" "byte" "unichar" "ubyte" "ushort" "ulong" "ullong"
	"slong" "sllong" "sint" "wchar_t"
	"complexfloat" "complexdouble" "complexlong" "size_t"
	"byte8" "uint16" "uint32" "uint64" "int16" "int32" "int64" 
	"float32" "float64"))

;; define our c++ syntactic guidelines for font-lock
;;
(font-lock-add-keywords 
 'c++-mode
 '(
   ;; mark anything with all lower-case letters and ending with _a or _d
   ;; as a variable (class data or arguments)
   ;;
   ("\\<\\([a-z0-9_]+_[ad]\\)\\>" 1 font-lock-variable-name-face)

   ;; define our general function name rule -- the name should start
   ;; with a lower-case character, have mixed-case characters in the
   ;; name, and contain no underscores
   ;;
   ("\\<\\([a-z][a-z]+[A-Z][a-zA-Z]+\\)\\>" 1 font-lock-function-name-face)

   ;; define some common single-word function names that won't be
   ;; picked up by our general capitalization rule
   ;;
   ("\\<\\(name\\|diagnose\\|debug\\|assign\\|read\\|write\\|eq\\|load\\|handle\\|open\\|close\\|get\\|clear\\|find\\|put\\|length\\|concat\\|insert\\|lt\\|gt\\|le\\|ge\\|ne\\|rand\\|hash\\|init\\|reset\\)\\>" 1 font-lock-function-name-face)
   
   ;; define our general type name/class name rule -- the name should
   ;; start with a capital letter but have some lower-case characters
   ;; in it
   ;;
   ("\\<\\([A-Z]+[a-z]+[a-zA-Z]+\\)\\>" 1 font-lock-type-face)

   ;; define our general constant rule -- all upper case characters
   ;; and allow underscores and numbers after the first upper case
   ;; character. it also allows the name to start with an arbitrary
   ;; number of underscores as long as there are no lower-case
   ;; characters
   ;;
   ("\\<\\(_*[A-Z][A-Z0-9_]+\\)\\>" 1 font-lock-constant-face)))

;;---------------------------------------------------------------------------;;
;;									     ;;
;; other code development related settings				     ;;
;;									     ;;
;;---------------------------------------------------------------------------;;

;; setup the jde java tools for compiling and debugging java files
;;
(add-hook 'java-mode-hook 
	  '(lambda () (require 'jde)))

;; make .html files automatically use html-helper mode
;;
(setq auto-mode-alist
      (cons (cons "\\.html$" 'html-helper-mode) auto-mode-alist))

;; make .h files automatically use c++ mode
;;
(setq auto-mode-alist
      (cons (cons "\\.h$" 'c++-mode) auto-mode-alist))
(setq c++-untame-characters nil)

;; make .sh files automatically use the shell mode
;;
(setq auto-mode-alist
      (cons (cons "\\.sh$" 'sh-mode) auto-mode-alist))

;; make .pl files automatically use the perl mode
;;
(setq auto-mode-alist
      (cons (cons "\\.pl$" 'perl-mode) auto-mode-alist))

;; set the default executable line for scripting languages
;;
;; We specifically quote the regular expresions for the file names
;; (rather than extracting it from the auto-mode-alist) because you
;; want the magic string added to only the .pl files, not the .pm
;; files (which are also opened in perl mode), etc.
;;
(add-hook 'perl-mode-hook 
	  (function 
	   (lambda ()
	     (if (string-match "\\.[Pp][Ll]\\'" (or (buffer-file-name) ""))
		 (executable-set-magic "/usr/local/bin/perl" "" t t)))))

;; disable this -- it breaks running wish easily
;;
;;(add-hook 'tcl-mode-hook 
;;	  (function 
;;	   (lambda ()
;;	     (if (string-match "\\.[Tt][Cc][Ll]\\'" (or (buffer-file-name) ""))
;;		 (executable-set-magic "/usr/local/bin/wish" "-f" t t)))))

;; disable this - it breaks bash files
;;
;;(add-hook 'sh-mode-hook 
;;	  (function 
;;	   (lambda ()
;;	     (if (string-match "\\.[Ss][Hh]\\'" (or (buffer-file-name) ""))
;;		 (executable-set-magic "/bin/sh" "" t t)))))

;; set the default compile command
;;
(setq compile-command "$ISIP_MAKE")

;; allow error messages to not be left justified
;;
(add-hook 'c++-mode-hook
	  '(lambda () 
	     (setq compilation-error-regexp-alist
		   (cons (list "\\ *\\([a-zA-Z][-a-zA-Z_0-9]+.cc: ?\\)?\\([a-zA-Z]?:?[^:( 	\n]*[^:( 	\n0-9][^:( 	\n]*\\)[:(][ 	]*\\([0-9]+\\)\\([) 	]\\|:\\(\\([0-9]+:\\)\\|[0-9]*[^:0-9]\\)\\)" 2 3 6) nil))))

;;---------------------------------------------------------------------------;;
;;									     ;;
;; special foreign language modes			     		     ;;
;;									     ;;
;;---------------------------------------------------------------------------;;
(if (boundp 'MULE) (setq jserver-list (list (getenv "ISIP_HOST"))))
(if (boundp 'MULE) (set-default-file-coding-system *euc-japan*))
(if (boundp 'MULE) (load-file "/common/mule/lisp/quail/latin.el"))

;;
;; end of file
