#! @SH@

# file: $isip/util/misc/isip_set_x_info/isip_set_x_info.sh
#
# this file primarily sets the keyboard mapping and font paths for x11.
# since these quantities are often terminal dependent, this function
# was localized to this file.
#
# 12/27/97 (joe picone): added a source of the environment for x11r6
# 			 this has been a constant battle - xrsh on r6 does not
#			 appear to pick up the environment
#
# 1/15/95 (joe picone): created this file when isip was established

# make sure there is an isip environment - must be done at the top
#
if (test -z "$ISIP") then
    . $HOME/.bashrc;
fi

# important help-related definitions
#
PROGRAM_NAME="isip_set_x_info";

ISIP_HELP_FILE="$ISIP/util/misc/isip_set_x_info/isip_set_x_info.help";
export ISIP_HELP_FILE;

# define vendor strings - ways to identify a terminal
#
DEFAULT_ISIP_VENDOR_NAME_SUN="X Consortium";
DEFAULT_ISIP_VENDOR_NAME_NCD="Network Computing Devices Inc.";
DEFAULT_ISIP_VENDOR_NAME_GRAPHON="MIT X Consortium";

# define keyboard mappings
#
DEFAULT_ISIP_XMODMAPRC_SUN="$ISIP/login/x11_xmodmaprc_sun.sh";
DEFAULT_ISIP_XMODMAPRC_NCD="$ISIP/login/x11_xmodmaprc_ncd.sh";
DEFAULT_ISIP_XMODMAPRC_GRAPHON="$ISIP/login/x11_xmodmaprc_graphon.sh";

# define font paths
#
DEFAULT_ISIP_X_FONTS_SUN="/usr/openwin/lib/X11/fonts/75dpi";
DEFAULT_ISIP_X_FONTS_NCD="/usr/local/ncd/usr/lib/X11/ncd/fonts/pcf/75dpi";
DEFAULT_ISIP_X_FONTS_GRAPHON="$XGOHOME/lib/X11/fonts/75dpi";

# execute a generic help function:
#  force the shellscript to exit if a help option is encountered.
set -e;
isip_function_help_0 $*;
set +e;

# try to figure out what type of terminal this is - xdpyinfo
# appears to be the most reliable way to do this.
#ISIP_VENDOR_NAME_SUN=`xdpyinfo | grep "$DEFAULT_ISIP_VENDOR_NAME_SUN"`;
#ISIP_VENDOR_NAME_NCD=`xdpyinfo | grep "$DEFAULT_ISIP_VENDOR_NAME_NCD"`;
#ISIP_VENDOR_NAME_GRAPHON=`xdpyinfo | egrep "$DEFAULT_ISIP_VENDOR_NAME_GRAPHON"`;

# modify x11 according to the terminal type:
#  since sun and graphon overlap, look for graphon first
#
#if (test "$ISIP_VENDOR_NAME_GRAPHON") then
##    xset +fp "$DEFAULT_ISIP_X_FONTS_GRAPHON";
#    xmodmap "$DEFAULT_ISIP_XMODMAPRC_GRAPHON";
#elif (test "$ISIP_VENDOR_NAME_NCD") then
##    xset +fp "$DEFAULT_ISIP_X_FONTS_NCD";
#    xmodmap "$DEFAULT_ISIP_XMODMAPRC_NCD";
#elif (test "$ISIP_VENDOR_NAME_SUN") then
##    xset +fp "$DEFAULT_ISIP_X_FONTS_SUN";
#    xmodmap "$DEFAULT_ISIP_XMODMAPRC_SUN";
#else
#    echo "error($PROGRAM_NAME): unknown terminal type";
#    exit $ISIP_ERROR;
#fi

# Try a manual xmodmap (can't get this to work from a file)
#
xmodmap -e "remove Lock = Caps_Lock"
xmodmap -e "remove Control = Control_L"
xmodmap -e "keysym Control_L = Caps_Lock"
xmodmap -e "keysym Caps_Lock = Control_L"
xmodmap -e "add Lock = Caps_Lock"
xmodmap -e "add Control = Control_L"

#
# end of file
