// file: $isip/util/speech/isip_verify/isip_verify.cc
// version: $Id: isip_verify.cc,v 1.2 2003/11/07 16:21:12 tang Exp $
//
// isip include files
//
#include "isip_verify.h"
#include "HierarchicalSearch.h"

// isip_verify: core part of speaker verification system,
// outputs binary decision
//
// it will be used for
//    speaker verification
//    speaker enrollment
//
int main(int argc, const char** argv) {

  // setup the command line
  //
  Sdb sdb;
  CommandLine cmdl(sdb);
  cmdl.setUsage(
#include "usage_message.text"
                );
  cmdl.setHelp(
#include "help_message.text"
               );

  cmdl.setIdent("$Revision: 1.2 $",
		"$Name: isip_r00_n11_t02 $",
		"$Date: 2003/11/07 16:21:12 $");
  
  // add a command line option for the parameters file
  //
  Filename params_file;
  cmdl.addOptionParam(params_file, OPTION_PARAMS_FILE, (Filename)EMPTY);

  // add a command line option for the accumulator file
  //
  Filename accum_file;
  cmdl.addOptionParam(accum_file, OPTION_ACCUMULATOR_FILE, (Filename)EMPTY);

  // add a command line option for the accumulator list
  //
  Filename accum_list;
  cmdl.addOptionParam(accum_list, OPTION_ACCUMULATOR_LIST, (Filename)EMPTY);
  
  // add a command line option for the output mode
  //
  String output_mode;
  cmdl.addOptionParam(output_mode, OPTION_OUTPUT_MODE, EMPTY);

  // add a command line option for the function mode
  //
  String function_mode;
  cmdl.addOptionParam(function_mode, OPTION_FUNCTION_MODE, EMPTY);  
  
  // add a command line option for the output format
  //
  String output_format;
  cmdl.addOptionParam(output_format, OPTION_OUTPUT_FORMAT, (Filename)EMPTY);

  // add a command line option for the output file
  //
  Filename output_file;
  cmdl.addOptionParam(output_file, OPTION_OUTPUT_FILE, (Filename)EMPTY);
  
  // add a command line option for the output list
  //
  Filename output_list;
  cmdl.addOptionParam(output_list, OPTION_OUTPUT_LIST, (Filename)EMPTY);

  // add a command line option for the language model file
  //
  Filename lm_file;
  cmdl.addOptionParam(lm_file, OPTION_LANGUAGE_MODEL_FILE, (Filename)EMPTY);

  // add a command line option for the statistical model pool file
  //
  Filename smp_file;
  cmdl.addOptionParam(smp_file, OPTION_STATISTICAL_MODEL_POOL_FILE,
		      (Filename)EMPTY);

  // add a command line option for the imposter language model file
  //
  Filename imp_lm_file;
  cmdl.addOptionParam(imp_lm_file, OPTION_IMPOSTER_LANGUAGE_MODEL_FILE,
		      (Filename)EMPTY);

  // add a command line option for the imposter statistical model pool file
  //
  Filename imp_smp_file;
  cmdl.addOptionParam(imp_smp_file,
		      OPTION_IMPOSTER_STATISTICAL_MODEL_POOL_FILE,
		      (Filename)EMPTY);

  // add a command line option for the language model output file
  //
  Filename lm_output_file;
  cmdl.addOptionParam(lm_output_file, OPTION_LANGUAGE_MODEL_OUTPUT_FILE,
		      (Filename)EMPTY);

  // add a command line option for the statistical model pool output file
  //
  Filename smp_output_file;
  cmdl.addOptionParam(smp_output_file,
		      OPTION_STATISTICAL_MODEL_POOL_OUTPUT_FILE,
		      (Filename)EMPTY);  

  // add a command line option for the debug level
  //
  DebugLevel debug_level;
  cmdl.addOptionParam(debug_level, OPTION_DEBUG_LEVEL,
		      DEBUG_LEVEL_DEFAULT);

  // add a command line option for the log verbosity level
  //
  DebugLevel verbosity;
  cmdl.addOptionParam(verbosity, OPTION_VERBOSITY_LEVEL,
		      VERBOSITY_LEVEL_DEFAULT);

  // add a command line option for the log file
  //
  Filename log_file;
  cmdl.addOptionParam(log_file, OPTION_LOG_FILE, (Filename)LOG_FILE_DEFAULT);
  
  // add verify command line option for output levels
  //
  String output_level;
  cmdl.addOptionParam(output_level, OPTION_OUTPUT_LEVEL, EMPTY);
  
  // add verify command line option for update levels
  //
  String update_level;
  cmdl.addOptionParam(update_level, OPTION_UPDATE_LEVEL, EMPTY);
  
  // add a command line option for the decision tree file
  //
  Filename dt_file;
  cmdl.addOptionParam(dt_file, OPTION_DECISION_TREE_FILE, (Filename)EMPTY);

  // add a command line option for the question and answer file
  //
  Filename qes_ans_file;
  cmdl.addOptionParam(qes_ans_file, OPTION_QUESTION_ANSWER_FILE, (Filename)EMPTY);  

  // add a command line option for merge thresholds
  //
  String merge_threshold;
  cmdl.addOptionParam(merge_threshold, OPTION_MERGE_THRESHOLD, EMPTY);
  
  // add a command line option for split thresholds
  //
  String split_threshold;
  cmdl.addOptionParam(split_threshold, OPTION_SPLIT_THRESHOLD, EMPTY);
  
  // add verify command line option for occupancy threshold
  //
  String occ_threshold;
  cmdl.addOptionParam(occ_threshold, OPTION_OCC_THRESHOLD, EMPTY);

  // add a command line option for decision thresholds
  //
  String decision_thresholds;
  cmdl.addOptionParam(decision_thresholds, OPTION_DECISION_THRESHOLDS, EMPTY);

  // add streaming input command line flag
  //
  Boolean stream;
  cmdl.addFlagParam(stream, OPTION_FLAG_STREAM);

  // add verify command line flag
  //
  Boolean verify;
  cmdl.addFlagParam(verify, OPTION_FLAG_VERIFY);
  
  // add a command line option for output identifier for hypotheses
  //
  Filename output_base_name;
  cmdl.addOptionParam(output_base_name, OPTION_OUTPUT_BASE_NAME,(Filename) EMPTY);

  // parse the command line
  //
  if (!cmdl.parse(argc, argv)) {
    cmdl.printUsage();
  };

  // open the log file
  //
  if (log_file.ne(L"-")) {
    Console::open(log_file);
    Console::setLineWrap(File::NO_WRAP);
  }

  // print the command line
  //
  if (verbosity >= Integral::BRIEF) {
    cmdl.printCommandLine();
    cmdl.printVersion();
    //HierarchicalSearch::setDebug(Integral::BRIEF);
  }

  // check the param file argument
  //
  if(params_file.eq((Filename)EMPTY)) {
    cmdl.printUsage();
  };

  // set up the speaker verifier
  //
  SpeakerVerifier sv;

  // read parameters from the parameter file
  //
  sv.setParam(params_file);

  // set parameters from the command line
  //
  sv.setStream(stream);
  sv.setVerify(verify);
  sv.setDebug(debug_level);
  sv.setVerbosity(verbosity);
  sv.setAccumulatorFile(accum_file);
  sv.setAccumulatorList(accum_list);
  sv.setOutputMode(output_mode);
  sv.setOutputFormat(output_format);
  sv.setOutputFile(output_file);
  sv.setOutputList(output_list);
  sv.setFunctionMode(function_mode);
  sv.setLanguageModelFile(lm_file);
  sv.setImposterLanguageModelFile(imp_lm_file);
  sv.setLanguageModelUpdateFile(lm_output_file);
  sv.setStatisticalModelPoolFile(smp_file);
  sv.setImposterStatisticalModelPoolFile(imp_smp_file);
  sv.setStatisticalModelPoolUpdateFile(smp_output_file);
  sv.setOutputLevelString(output_level);
  sv.setUpdateLevelString(update_level);  
  
  // decision tree parameters
  //
  sv.setDecisionTreeFile(dt_file);
  sv.setQuesAnswerFile(qes_ans_file);
  sv.setSplitThreshold(split_threshold);
  sv.setMergeThreshold(merge_threshold);
  sv.setNumOccThreshold(occ_threshold);

  // decision thresholds
  //
  sv.setDecisionThresholds(decision_thresholds);

  // if streaming input, there must either be list or base name provided
  //
  if (stream) {

    // if no list file was provided, sdb is empty.
    // in which case, we must have a base name
    //
    if (!sdb.gotoFirst()) {

      // adding base name if users specified
      //
      if (output_base_name.length() > 0) {
	sdb.append(output_base_name);
      }

      // if no base name or list file, error out.
      //
      else {
	cmdl.printUsage();
      }
    }
    
  }
  
  // run the speech recognizer
  //
  sv.run(sdb);
  
  // close the log file
  //
  if (log_file.ne(L"-")) {
    Console::close();
  }

  // exit gracefully
  //
  return Integral::exit();
}
